function enrgtot= ComputeEnergyMC(efrkd,params,data)
% enrgtot= ComputeEnergyMC(efrkd,params,data)
% This function computes energy from the likelihood function 
% (i.e. not including prior) energy= -Log[P(data|params)]
% Computation includes marginalization of D0,A0
% INPUT:
%   1. An Efr-Kd pair (2x1 vector)
%   2. cparams (photophys constants relating concentration to brightness)
%       Can be 3x3 matrix in a structure or just the matrix itself
%   3. data: A structure of length m (number of samples) is expected. For
%       each sample there should be an nx3 matrix, ydata (n meas., 3 channels). 
% OUTPUT:
%   1. enrgtot= sum(enrg)
%   enrg (mx1 vec) is the energy integrated over A0,D0. (enrg=-logPint)
%   logPint (mx1 vec)= log of marginal likelihood
%
% C. Lichten, University of Edinburgh, 2010
% http://swainlab.bio.ed.ac.uk/software.php


if length(efrkd)==11
    cparams= reshape(efrkd(3:11),3,3);
elseif isstruct(params)
    cparams= params.cparams;
else
    cparams= params;
end

persistent optD0A0 %To reuse optD0A0 when fun is called on multiple measurements
if isempty(optD0A0)
    optD0A0= 1e-4*ones(length(data),2);
end

enrg= ones(length(data),1); optE= ones(size(enrg)); logPint= ones(size(enrg));
for i= 1:length(data)
    [enrg(i),optD0A0(i,:),optE(i),logPint(i)]= ...
        ComputeEnergy(efrkd,cparams,data(i),optD0A0(i,:));
end
enrgtot= sum(enrg);

if imag(enrgtot)~=0
    enrgtot= Inf;
end

function [enrg,optD0A0,optE,logPint]= ComputeEnergy(efrkd,cparams,data,optD0A0prev) 

options= optimset('Display','Off');
% here we find the optimal D0,A0 by minimizing (dE/dD0)^2+(dE/dA0)^2
enrgfun=@(D0A0) Enrgd2D0A0(D0A0,efrkd,cparams,data);

[optD0A0,f]= fminsearch(enrgfun,optD0A0prev,options); 
% Test many x0 to improve result if energy is imaginary OR if this is the 
% first run (optD0A0prev==1e-4) OR if A0<0 or D0<0:
if (imag(f)~=0 || all(optD0A0prev(:,1)==1e-4) || any(optD0A0<0))
    [optD0A0(2,:),f(2)]= fminsearch(enrgfun,[1e-8 1e-8],options);
    [optD0A0(3,:),f(3)]= fminsearch(enrgfun,[1e-6 1e-6],options);
    [optD0A0(4,:),f(4)]= fminsearch(enrgfun,[1e-4 1e-4],options);
    [optD0A0(5,:),f(5)]= fminsearch(enrgfun,[1e-2 1e-2],options);
    [optD0A0(6,:),f(6)]= fminsearch(enrgfun,[1e2 1e2],options);
    [optD0A0(7,:),f(7)]= fminsearch(enrgfun,[1e4 1e4],options);
    [optD0A0(8,:),f(8)]= fminsearch(enrgfun,[1e6 1e6],options);
end

% Find the best optD0A0 from the ones found above:
if ~isempty(f(imag(f)==0))
    best= find(f==min(f(imag(f)==0)));
    best= best(1);
else
    best= find(f==min(f));
    if ~isempty(best); best= best(1); else best=1; end
end
optD0A0= optD0A0(best,:);  optE= f(best);

%Then we find the value of the second derivs of the Enrg wrt D0,A0 at those
%optimal values, and dh= det(hessian)= E2dd*E2aa-E2ad^2
[temp,E2dd,E2aa,E2ad]= Enrgd2D0A0(optD0A0,efrkd,cparams,data);
dh= E2dd*E2aa-E2ad^2; clear temp

% for a min, dh should be positive. If it isn't, use numerical method
% for getting the hessian: (DERIVestSuite code)
if dh<0
    h= hessian(enrgfun,optD0A0);  
    dh=det(h);
end

% probability -> energy
logPint= log(2*pi) - optE - log(dh)/2;
enrg= -logPint;

if imag(enrg)~=0 %Give warning if value for enrg is still imaginary
    fprintf('Warning, energy imaginary for D0=%4.1f, A0=%4.1f, log10(Kd)=%2.2f, Efr=%1.2f. \n' ...
        ,optD0A0(1),optD0A0(2),efrkd(2),efrkd(1));
end
