function ShowFigures(DRAMresults,data)
% Function makes plots to visualize results from MCMC walk and 
%   energy contour plot
% INPUT:
%   1. DRAMresults= MCMC walk and energy contour
%   2. data= structure of data
%
% Auto-generated by MATLAB, subsequently edited by 
% C. Lichten, University of Edinburgh, 2010
% http://swainlab.bio.ed.ac.uk/software.php

addpath('utils/');
addpath('utils/mcmcstat/');

%% 1. Assign variables
%  CHAIN1:  vector of x data from MCMC walk (Efr)
%  CHAIN2:  vector of y data from MCMC walk (log10kd)
%  XDATA1:  contour x values (Efr)
%  YDATA1:  contour y values (log10kd)
%  ZDATA1:  contour z values (enrgcon')
chain1= DRAMresults.chain(:,1); chain2= DRAMresults.chain(:,2);
xdata1= DRAMresults.Efr;  ydata1= DRAMresults.log10kd;
zdata1= DRAMresults.enrgcon';
Enrg= DRAMresults.energy;
trueEfr= data.Efr;  trueKd= data(1).kd_eff;

%% 2. Plot Energy contour with MCMC walk trail on top
% Create figure
figure1 = figure('NumberTitle','off','XVisual',...
    '0x24 (TrueColor, depth 24, RGB mask 0xff0000 0xff00 0x00ff)',...
    'Name','Energy contour with walk','Position',[0,0,600,500]);

% Create axes
axes1 = axes('Parent',figure1,'Layer','top');
% Uncomment the following line to preserve the X-limits of the axes
xlim(axes1,[min([xdata1(1) min(chain1)]), max([xdata1(end) max(chain1)])]);
% Uncomment the following line to preserve the Y-limits of the axes
ylim(axes1,[min([ydata1(1) min(chain2)]), max([ydata1(end) max(chain2)])]);
box(axes1,'on');
hold(axes1,'all');

% Create contour
contourf(xdata1,ydata1,zdata1,50,'LineColor',[0 0 0],'Fill','on');

% Create xlabel
xlabel('FRET Efficiency (E_{fr})','FontSize',14,'FontWeight','demi');

% Create ylabel
ylabel('Log_{10}(Kd)','FontSize',14,'FontWeight','demi');

% Create title
title('Energy Contour with random walk on top  ','FontSize',14,'FontWeight','bold');

% Create plot
plot(chain1,chain2,'.','Color',[0.6784 0.9216 1],'DisplayName','chain');

% Create plot
plot(chain1(1),chain2(1),'MarkerFaceColor',[1 0 0],'MarkerEdgeColor',[1 0 0],...
    'MarkerSize',15,...
    'Marker','x',...
    'LineStyle','none',...
    'DisplayName','chain start');

% Create plot
plot(chain1(end),chain2(end),'MarkerFaceColor',[1 0 0],'MarkerEdgeColor',[1 0 0],...
    'MarkerSize',15,...
    'Marker','x',...
    'LineStyle','none',...
    'DisplayName','chain end');

% Create textarrow 
% [arrow start(x), arrow end(x)],[arrow start(y), arrow end(y)]
[x,y]= dsxy2figxy([chain1(1),chain1(1)],[chain2(1),chain2(1)]);
annotation(figure1,'textarrow',[x(1)-.06,x(2)-.02],[y(1)-.06,y(2)-.02],... 
    'TextEdgeColor',[0.749 0 0.749],...
    'TextLineWidth',0.5,...
    'TextColor',[0.749 0 0.749],...
    'TextBackgroundColor',[1 1 1],...
    'FontSize',14,...
    'String',{'start',num2str([chain1(1) chain2(1)],'%3.2g %3.2g')},...
    'Color',[0.749 0 0.749]);

% Create textarrow
[x,y]= dsxy2figxy([chain1(end),chain1(end)],[chain2(end),chain2(end)]);
annotation(figure1,'textarrow',[x(1)-.06,x(2)-.02],[y(1)-.06,y(2)-.02],... 
    'TextEdgeColor',[0.749 0 0.749],...
    'TextLineWidth',0.5,...
    'TextColor',[0.749 0 0.749],...
    'TextBackgroundColor',[1 1 1],...
    'FontSize',14,...
    'TextColor',[0.749 0 0.749],...
    'String',{'end',num2str([chain1(end) chain2(end)],'%3.2g %3.2g')},...
    'Color',[0.749 0 0.749]);

% Create textarrow
plot(trueEfr,log10(trueKd),'gx','MarkerSize',15)
[x,y]= dsxy2figxy([trueEfr,trueEfr],log10([trueKd,trueKd]));
annotation(figure1,'textarrow',[x(1)-.06,x(2)-.02],[y(1)-.06,y(2)-.02],... 
    'TextEdgeColor',[0 0.7 0],...
    'TextLineWidth',0.5,...
    'TextColor',[0 0.7 0],...
    'TextBackgroundColor',[1 1 1],...
    'FontSize',14,...
    'TextColor',[0 0.7 0],...
    'String',{'True Value',num2str([trueEfr log10(trueKd)],'%3.2g %3.2g')},...
    'Color',[0 .7 0]);

%% 3. Plot ellipses showing clustering of steps and smoothed histograms of
%   steps

figure('NumberTitle','off','Name','Efr-Kd ellipse and histograms',...
    'Position',[580,0,500,400]);
xlim([xdata1(1),xdata1(end)]),ylim([ydata1(1) ydata1(end)])
mcmcplot([chain1(1000:end) chain2(1000:end)],[],[],'kdefr',3)
ylabel('Log_{10}(K_d)','FontSize',14,'FontWeight','demi')
xlabel('E_{fr}','FontSize',14,'FontWeight','demi')

%% 4. Plot energy vs. steps taken
figure('NumberTitle','off','Name','Energy evolution during walk',...
    'Position',[0,600,500,250]);
plot(Enrg), ylabel('Energy','FontSize',14,'FontWeight','demi')
xlabel('Steps','FontSize',14,'FontWeight','demi')

%% 5. Plot parameters vs. steps taken
figure('NumberTitle','off','Name','Parameter evolution during walk',...
    'Position',[1050,0,600,950]);
subplot(2,1,1)
plot(chain1,'.');ylabel(['E_{fr}, initial value= ' num2str(chain1(1))]);
hold on, plot([1,length(chain1)],[trueEfr trueEfr],'r--');
title(sprintf('%s random walk. Acceptance rate: %.1f%%',upper(DRAMresults.method),...
    DRAMresults.accepted*100)), xlabel('Steps')
subplot(2,1,2)
plot(chain2,'.');ylabel(['Log_{10}(K_d), initial value= ' num2str(chain2(1))]);
hold on, plot([1 length(chain1)],log10([trueKd trueKd]),'r--');
xlabel('Steps')

%% 6. Plot histograms of Kd and Efr
figure, subplot(2,1,1), hist(chain1,-0.1:.01:1.1)
xlabel('E_{fr}','FontSize',14,'FontWeight','demi') 
ylabel('P(E_{fr}|data)','FontSize',14,'FontWeight','demi')
title('Posterior Distributions','FontSize',14,'FontWeight','bold')
subplot(2,1,2), hist(chain2,100)
xlabel('log_{10}(K_d)','FontSize',14,'FontWeight','demi') 
ylabel('P(log_{10}(K_d)|data)','FontSize',14,'FontWeight','demi')
