% find_kdefr.m
% This script calls code to run the Metropolis Hastings algorithm 
% (or delayed rejection, adaptive metropolis [DRAM] options)
% It seeks to optimize [efr,kd] (and other parameters) for a given set of
% three-cube FRET data
% 
% Adapted from ABrun.m from www.helsinki.fi/~mjlaine/dram/
% C. Lichten, University of Edinburgh, 2010
% http://swainlab.bio.ed.ac.uk/software.php

tic

% Clear persistent variables and other variables
clear ComputeEnergyMC model params options

% Select method to use
method = 'mh'; % 'mh','am','dr', or 'dram', see below

N= size(data(1).ydata,1);
rss= 1000;
if vary_all==1 % vary all parameters and start at converged efrkd
    x0= [startingValues reshape(cparams,1,9)]; %[.2 -4 reshape(cparams,1,9)]; %
elseif vary_all==-1 % only vary kd,efr but start at converged efrkd
    x0= startingValues; 
elseif vary_all==0 % only vary kd,efr
    x0= [rand -11+16*rand];
elseif vary_all==2  % vary all parameters, starting at random efrkd
    x0= [rand -11+16*rand reshape(cparams,1,9)];
end

p = length(x0);
mse = rss/N;%/(n-p); % mean squared error

% J'*J is numerically singular, so increase the smallest singular value
%  [u,s,v]=svd(J); s=diag(s);s(s<1e-3)=1e-3;
% covariance matrix of the lsq estimates, used as initial proposal
eyep= eye(p); eyep(1,1)= 1e-2; eyep(2,2)= 1; sig= .0001;
if vary_all>0
    eyep(3:11,3:11)= sig*eye(9);
end
cmat=cv*eyep;% 5e-3*eyep;

% Parameters for MCMC methods
switch method
  case 'mh'
    nsimu    = NumSteps; % make nsimu >1000
    drscale  = 0;
    adaptint = 0;
  case 'dr'
    nsimu    = 16000;
    drscale  = 2;
    adaptint = 0;
  case 'am'
    nsimu    = 16000;
    drscale  = 0;
    adaptint = 2000;
  case 'dram'
    nsimu    = 16000;
    drscale  = 2;
    adaptint = 2000;
end

% Create input arguments for the dramrun function:
model.ssfun    = @ComputeEnergyMC;
if vary_all>0
    model.priorfun = @priorfun;
else 
    model.priorfun= @priorfun;
end

params.par0    = x0; % initial parameter values
params.n       = N;  % number of observations
params.sigma2  = 1;  %rss;  % prior for error variance sigma^2
params.n0      = -1;  % prior accuracy for sigma^2 (no update if <0)
params.cparams = cparams; %%% rates for FRET fitting %%%
params.cmat      = cmat;

% Define Prior
if isempty(parmu0)
    params.parmu0   = x0; %prior mean 
    params.parsig0= 1e4*ones(1,length(x0)); % prior std 
else
    params.parmu0= parmu0; 
    params.parsig0= parsig0;
end
if vary_all>0 % varying Kd,Efr, and other parameters
    params.bounds= [-0.1 -11 -10*ones(1,9) ;1.1 5 100*ones(1,9)];
else % varying only Kd and Efr
    params.bounds= [-0.1 -12 ;1.1 0];
end

options.nsimu    = nsimu;          % size of the chain
options.adaptint = adaptint;       % adaptation interval
options.drscale  = drscale;
options.qcov     = cmat.*2.4^2./p; % initial proposal covariance 
% options.printint= 1000;

%% run MCMC- generate the chain (random walk)
[DRAMresults,chain,s2chain] = dramrun(model,data,params,options);
DRAMresults.chain= chain;
DRAMresults.s2chain= s2chain;
DRAMresults.method= method;
el_time= toc;
fprintf('\n Walk finished in %6.2f minutes. \n ',el_time/60);
DRAMresults.elapsedTime= el_time;
DRAMresults.params= params;
DRAMresults.options= options;
DRAMresults.model= model;

